## Modified for tidyverse from Helen Lockstone's RNA-Seq script - by Shoumo Sep 2020
## all data provided in metadata, no script changes are necessary. Read and checked by Helen
## and minor changes made.

#---DIRECTORIES NEEDED BEFORE YOU START ------
# 1 ./metadata/
# 2 ./data./"projectname"/
# 3 ./analyses/

#---FILES NEEDED BEFORE YOU START ------
# 1. metadata file (metadata.xlsx) - location: ./metadata./metadata.xlsx
# 2. layout file (layout.csv) - location: ./data/"projectname"/layout.csv
      #"projectname is specified in the metadata.xlsx
      #ensure that all column headers begin in lowercase
      #for data from GEO use the GEO script to generate this
# 3. gene count file (gene_count_table.txt) - location: ./data/"projectname"/gene_count_table.txt
      #column header for the gene_count_table.txt should label the first column "id"
      #the other column names should match either the index or the name column in layout.csv

#---FILES CREATED BY THIS SCRIPT APPEAR IN ./analysis/"projectname"/ --------
# 1. "projectname"_DE_plot_edgeR.pdf
# 2. "projectname"_edgeR_results_annotated.txt

## load software
library(tidyverse)
require(edgeR)
library(biomaRt)

#get the metadata file and create a subdirectory for depositing the results
metadata <- readxl::read_excel("./metadata/metadata.xlsx")
dir.create(paste0("./analyses/",metadata$projectName))
writepath <- paste0("./analyses/",metadata$projectName, "/")

#read in gene count data.
genecount_df <- read_delim(paste0("./data/",metadata$projectName,"/gene_count_table.txt"), delim="\t")

#read in layout. 
layout_df <- read_csv(paste0("./data/",metadata$projectName,"/layout.csv"))

#order by experiment group
layout_df <- layout_df %>%  arrange(group)

#get column names.This matches the correct layout column to the column header in gene_counts.
examplecolname <- names(genecount_df)[2]
colid <- which (grepl(examplecolname, layout_df))


#filter for the correct test and control samples  - this info is provided in the metadata file.
layout_df <- layout_df %>% filter(group == metadata$control | group == metadata$test )
colnam <- pull(layout_df, colid)
#create a sample description
layout_df <- layout_df %>% unite("sampleDescription", "group", "rep", remove = F)

#select correct data from gene count using the ordered names column in layout
#this step also reorders the columns as needed for further manipulation
genecount_df2 <- genecount_df %>% 
  dplyr::select(id, all_of(colnam))
genecount_df3 <- genecount_df2 %>%
  column_to_rownames(var="id") 

#replace the colnames with the description, makes it easier to follow
names(genecount_df3)<- layout_df$sampleDescription

d <- as.matrix(genecount_df3) # this is needed for the edgeR DGEList function

#from layout_df get the smallest group size
group_sizes <- layout_df %>% 
  group_by(group) %>% 
  tally()
mingroupsize <- min(group_sizes$n) 
comparison <- paste0(max(group_sizes$n), "x",min(group_sizes$n))

## define the experimental group factor. Note that the control group should
# always be the reference, i.e. in th eFIRST column.
conds <- factor(layout_df$group, levels = c(metadata$control, metadata$test))
table(conds) 

######################################################################
## running edgeR for a given dataset d
y <- DGEList(counts=d, genes=row.names(d))
#head(y$counts)
#head(cpm(y)) 

#cpm is counts normalized to a column total of a million.

## check the number of reads (in millions) for all samples
#round(apply(d, 2, sum)/1000000, 1)
     
##calculate the cpm threshold from the read threshold of 10 (figure provided in metadata)
meanreadspersample <- sum(genecount_df3)/ncol(genecount_df3)
cpmthreshold <- metadata$readThreshold*1E6/meanreadspersample
cpmthreshold <- round(cpmthreshold, 2)

# keep genes with reasonable number of reads in the smallest group size
keep <- rowSums(cpm(y)> cpmthreshold ) >= mingroupsize

## extract the genes passing the expression level filter
y <- y[keep,]
dim(y)

y$samples$lib.size <- colSums(y$counts)
y$samples$lib.size

## TMM normalisation
y <- calcNormFactors(y)
y$samples

## sample clustering plot
plotMDS(y)

design <- model.matrix(~conds)
row.names(design) <- colnames(y)
colnames(design) <- gsub("conds", "", colnames(design))
head(design) 

design
# the design matrix specifies the group each sample belongs to 
# it is vital the samples are listed in the same order in the rows 
#of the design matrix and the columns of the count table 
#(this can be checked again but will be the case 
#because we ensured same order at earlier step

identical(row.names(design), colnames(y$counts)) #TRUE

## the next commands compute dispersion (variance) estimates for 
#(i) all genes 
#(ii) trended by expression level and 
#(iii) per gene 
## the individual gene dispersion estimates are used in the final step, 
#testing each gene for differential expression between groups. 
#Particularly for small datasets, the gene-wise estimates benefit from being adjusted 
#towards the trended estimates so they are more robust (I think edgeR does this behind the scenes). 

## estimate overall dispersion
y <- estimateGLMCommonDisp(y, design, verbose=T)
#Disp = 0.02844 , BCV = 0.1686

## esimate gene-wise dispersion
y <- estimateGLMTrendedDisp(y, design)
y <- estimateGLMTagwiseDisp(y, design)

## plot to visualise the mean-variance relationship for all genes 
#(expression level increases along x-axis). 
#In well-behaved datasets this will decrease and then flatten out from left to right. 
plotBCV(y)

## testing for differential expression
##  edgeR implements a negative bionomial model, which is appropriate for count data and captures the biological variability that exists between samples. 
#Using a generalized linear model (GLM) framework enables edgeR to handle simple through to very complex experimental designs and covariates easily - 
#we have a simple 2 group comparison here. 
#The next 2 commands fit the model and test for genes that are differentially expressed between the 2 conditions.
fit <- glmFit(y, design)

## likelihood ratio testing - by default the last coefficient in design matrix 
#is tested for significance, which in this case compares the 2 groups, 
#so we don't need to specify anything else here. 
#It is testing whether there is a significantly better fit 
#under the specified model (i.e. with the experimental groups defined) 
#compared to the null model (assuming all samples from one group). 
#For genes that are differentially expressed, 
#the fit will be far better and result in a high likelihood ratio, 
#from which the corresponding p-value is computed. 
res <- glmLRT(fit) 

## summarise results
## the function below extracts the results of the analysis, 
#adjusts the raw p-values for multiple testing, and formats them for output
## note a function is not necessarily required as we only have one comparison in this example
#but the code is used for other projects where multiple comparisons may be performed 
#and it becomes useful in that scenario. 

summarise.res <- function(res, project, comp.name)
{

	o <- order(res$table$PValue) 
	#res$table contains 4 columns, logFC (log fold change), logCPM (average log2-counts-per-million), LR (likelihood ratio), P value. 
	#and gene names as row name. The above command orders res$table by P value. in ascending prder.
	# 'o' simply has the row number
	#convert res$table to dataframe using restable <- as.data.frame(res$table)) to check.
	
	cpm(y)[o[1:10],] 
	# takes the first 10 rows of cpm (y) - with the lowest P values

	#Identify which genes are significantly differentially expressed
	#output is -1, 0 or 1
	de <- decideTestsDGE(res)

	## DE plots, highlighting sig genes
	detags <- rownames(y)[as.logical(de)]
   
	pdf(paste0(writepath, project,"_", metadata$subgroup, "_DE_plot_edgeR_", comp.name, ".pdf"))
	plotSmear(res, de.tags=detags, main=paste0(comp.name), ylim=c(-10,10))
	abline(h=c(-1,1), col="blue")
	dev.off()

	##res$table contains 4 columns, logFC, logCPM, LR, P value. The next command
	#orders it by ascending P value. i.e. most sig at top of list
    out <- res$table[o,]
	
	## add adjusted p-values
	adjp <- p.adjust(out$PValue, method="fdr")
	
	out <- cbind(out, adjp)
	# head(out)

	write.table(out, file=paste0(writepath, project, "_edgeR_results_", comp.name, ".txt"), 	sep="\t", quote=F, row.names=T)
    ## results at 5% fdr
    summary(de <- decideTestsDGE(res))
  
  	print(length(which(out$adjp<0.05)))

	return(out)
}

res <- summarise.res(res, metadata$projectName, comparison) # 3108

## after the line immediately above is run (executing the function defined above), 
#2 output files will be written to the current working directory - 
#a text file of the list of ALL genes - without any filtering
#and a summary plot.
#a count of differe

## annotate genes using biomaRt package

ensembl <- useMart("ensembl")

#listDatasets(ensembl). the metadata needs to contain which ensenml database to use.

ensembl <- useDataset(metadata$ensembl, mart=ensembl)
	
## filters define a restriction on the query e.g. Chromosome_name=1
filters <- listFilters(ensembl)
filters[1:5,]

## attributes define the values to retrieve with the query
attributes <- listAttributes(ensembl)
attributes[1:20,]

#this places the annotation into columns 6-14 of the previously printed res file
annotate <- function(file)
{
      res <- read.table(paste(file), sep="\t", header=T, row.names=1)
      ids <- row.names(res)

      annot <- getBM(attributes=c("ensembl_gene_id", "description", "external_gene_name", "entrezgene_id", "hgnc_symbol", "chromosome_name", "start_position", "end_position", "strand"), filters="ensembl_gene_id", values=ids, mart=ensembl)

      res[,6:14] <- annot[match(row.names(res), annot[,1]),]
      res <- cbind(row.names(res), res)
      colnames(res)[1] <- "Ensembl_ID"
      
      ## remove the duplicate ID column
      res <- res[, -(grep("ensembl_gene_id", colnames(res)))]
	  ## as mouse genome, also remove the hgnc_symbol column
	  res <- res[, -(grep("hgnc_symbol", colnames(res)))]
      return(res)
}

res.annot <- annotate(paste0(writepath, metadata$projectName, "_edgeR_results_", comparison, ".txt"))

dim(res.annot) # 11165   13

 ## add counts for samples in comparison - this is useful for interpreting the final gene list 
##and see how each gene is behaving in the individual samples and overall expression level, which are not evident from the summary statistics output from the edgeR analysis. 
#genecount_df2 has the columns sorted by group already
genecount_df4 <- genecount_df3 %>% rownames_to_column()

res.annot_df <- as_tibble(res.annot) %>% 
  left_join(genecount_df4, by = c("Ensembl_ID" = "rowname"))
#this file can be used for subsequent analysis.  It is typically filtered by adjp<0.05 before use.
write.table(res.annot_df, file= paste0(writepath, metadata$projectName, "_", metadata$subgroup, "_edgeR_results_", comparison, "_annotated.txt"), sep="\t", quote=F, row.names=F)
#remove the unannotated file
file.remove(paste0(writepath, metadata$projectName, "_edgeR_results_", comparison, ".txt"))




